#*******************************************************************************
#
# MIT License
#
# Copyright (c) 2025 Advanced Micro Devices, Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#*******************************************************************************
import os
import re
import logging

logging.basicConfig(level=logging.INFO, format='%(levelname)s: %(message)s')

FOLDER_PATH = "../../test/gtest"

# Ignore list: Add test names or file paths you want to exclude
#              (keep the list sorted to min. git conflicts)
IGNORE_LIST = {
    "../../test/gtest/binary_tensor_ops.cpp",
    "../../test/gtest/graphapi_conv_bias_res_add_activ_fwd.cpp",
    "../../test/gtest/graphapi_operation_rng.cpp",
    "../../test/gtest/layout_transpose.cpp",
    "../../test/gtest/reduce_custom_fp32.cpp",
    "../../test/gtest/unary_tensor_ops.cpp",
    "CPU_MIOpenDriverRegressionBigTensorTest_FP32",
    "GPU_UnitTestConvSolverAsmBwdWrW3x3Wrw_FP32",
}

# Valid enums and Regex for validation
VALID_HW_TYPES = {"CPU", "GPU"}
VALID_DATATYPES = {"FP8", "FP16", "FP32", "FP64", "BFP16", "BFP8", "I64", "I32", "I16", "I8", "NONE"}
# Our suite (or fixture) naming convention: must start with CPU or GPU, followed by one or more alphanum groups, and end with a valid datatype.
TESTSUITE_REGEX = re.compile(
    r"^(CPU|GPU)_[A-Za-z0-9]+(?:_[A-Za-z0-9]+)*_(" + "|".join(VALID_DATATYPES) + r")$"
)
# Test type for instantiations must be one of these words, optionally with extra alphanum characters.
TEST_TYPE_REGEX = re.compile(r"^(Smoke|Full|Perf|Unit)([A-Za-z0-9]*)?$")

# Updated regexes that do not allow newlines in the macro arguments
TEST_P_REGEX = re.compile(
    r"\bTEST_P\(\s*([^\n,]+?)\s*,\s*([^\n\)]+?)\s*\)"
)
INSTANTIATE_TEST_REGEX = re.compile(
    r"\bINSTANTIATE_TEST_SUITE_P\(\s*([^\n,]+?)\s*,\s*([^\n,]+?)\s*,"
)
ALLOW_UNINSTANTIATED_REGEX = re.compile(
    r"\bGTEST_ALLOW_UNINSTANTIATED_PARAMETERIZED_TEST\(\s*([^\n\)]+?)\s*\)"
)
TEST_REGEX = re.compile(
    r"\bTEST\(\s*([^\n,]+?)\s*,\s*([^\n\)]+?)\s*\)"
)
TEST_F_REGEX = re.compile(
    r"\bTEST_F\(\s*([^\n,]+?)\s*,\s*([^\n\)]+?)\s*\)"
)


def analyze_tests(folder_path):
    errors = []

    # Walk over all files in the folder
    for root, _, files in os.walk(folder_path):
        for file in files:
            if not file.endswith(".cpp"):
                continue

            file_path = os.path.join(root, file)

            # Skip file if it is in the ignore list
            if file_path in IGNORE_LIST:
                logging.info(f"Skipping ignored file: {file_path}")
                continue

            with open(file_path, "r") as f:
                content = f.read()

            # Use the content for proper line-number computation
            def get_line_number(position):
                # Count the number of newline characters before the position; add 1 so that numbering starts at 1.
                return content.count("\n", 0, position) + 1

            # Dictionaries to record macro definitions.
            # For TEST_P, TEST, and TEST_F we key on (suite_or_fixture, test_name)
            test_p_definitions = {}  # key: (suite, test_name) -> line number
            test_definitions = {}    # key: (suite, test_name) -> line number
            test_f_definitions = {}  # key: (fixture, test_name) -> line number

            # For INSTANTIATE_TEST_SUITE_P we group by test suite; for each suite, instantiation names must be unique.
            instantiations = {}  # key: suite -> dict of {instantiation_name: line number}

            # For GTEST_ALLOW_UNINSTANTIATED_PARAMETERIZED_TEST, we store a list of line numbers per suite.
            allowed_uninstantiated = {}  # key: suite -> list of line numbers

            # === Process TEST_P macros ===
            for m in TEST_P_REGEX.finditer(content):
                suite, test_name = m.groups()
                line = get_line_number(m.start())
                if suite in IGNORE_LIST or test_name in IGNORE_LIST:
                    logging.info(f"Skipping ignored test suite: {suite}")
                    continue
                if not TESTSUITE_REGEX.match(suite):
                    errors.append(f"{file_path}:{line}: Invalid TESTSUITE_NAME '{suite}' in TEST_P.")
                key = (suite, test_name)
                if key in test_p_definitions:
                    prev_line = test_p_definitions[key]
                    errors.append(
                        f"{file_path}:{line}: Duplicate TEST_P for '{suite}.{test_name}' (previously defined at line {prev_line})."
                    )
                else:
                    test_p_definitions[key] = line

            # === Process TEST macros ===
            for m in TEST_REGEX.finditer(content):
                suite, test_name = m.groups()
                line = get_line_number(m.start())
                if suite in IGNORE_LIST or test_name in IGNORE_LIST:
                    logging.info(f"Skipping ignored test suite: {suite}")
                    continue
                if not TESTSUITE_REGEX.match(suite):
                    errors.append(f"{file_path}:{line}: Invalid TEST suite name '{suite}' in TEST.")
                key = (suite, test_name)
                if key in test_definitions:
                    prev_line = test_definitions[key]
                    errors.append(
                        f"{file_path}:{line}: Duplicate TEST for '{suite}.{test_name}' (previously defined at line {prev_line})."
                    )
                else:
                    test_definitions[key] = line

            # === Process TEST_F macros ===
            for m in TEST_F_REGEX.finditer(content):
                fixture, test_name = m.groups()
                line = get_line_number(m.start())
                if fixture in IGNORE_LIST or test_name in IGNORE_LIST:
                    logging.info(f"Skipping ignored test fixture: {fixture}")
                    continue
                if not TESTSUITE_REGEX.match(fixture):
                    errors.append(f"{file_path}:{line}: Invalid TEST_F fixture name '{fixture}'.")
                key = (fixture, test_name)
                if key in test_f_definitions:
                    prev_line = test_f_definitions[key]
                    errors.append(
                        f"{file_path}:{line}: Duplicate TEST_F for '{fixture}.{test_name}' (previously defined at line {prev_line})."
                    )
                else:
                    test_f_definitions[key] = line

            # === Process INSTANTIATE_TEST_SUITE_P macros ===
            for m in INSTANTIATE_TEST_REGEX.finditer(content):
                instantiation_name, suite = m.groups()
                line = get_line_number(m.start())
                if suite in IGNORE_LIST or instantiation_name in IGNORE_LIST:
                    logging.info(f"Skipping ignored instantiation for suite: {suite}")
                    continue

                normalized_instantiation = instantiation_name.replace("\\", "").strip()
                if not TEST_TYPE_REGEX.match(normalized_instantiation):
                    errors.append(
                        f"{file_path}:{line}: Invalid TEST_TYPE '{instantiation_name}' in INSTANTIATE_TEST_SUITE_P."
                    )

                # For each suite, the instantiation names must be unique.
                if suite not in instantiations:
                    instantiations[suite] = {}
                if instantiation_name in instantiations[suite]:
                    prev_line = instantiations[suite][instantiation_name]
                    errors.append(
                        f"{file_path}:{line}: Duplicate INSTANTIATE_TEST_SUITE_P instantiation name '{instantiation_name}' for suite '{suite}' (previously defined at line {prev_line})."
                    )
                else:
                    instantiations[suite][instantiation_name] = line

            # === Process GTEST_ALLOW_UNINSTANTIATED_PARAMETERIZED_TEST macros ===
            for m in ALLOW_UNINSTANTIATED_REGEX.finditer(content):
                suite = m.group(1).strip()
                line = get_line_number(m.start())
                if suite in IGNORE_LIST:
                    logging.info(f"Skipping ignored allowed uninstantiated suite: {suite}")
                    continue
                allowed_uninstantiated.setdefault(suite, []).append(line)

            # === Validate TEST_P definitions have a corresponding instantiation or allowed exception ===
            for (suite, test_name), line in test_p_definitions.items():
                if suite not in instantiations and suite not in allowed_uninstantiated:
                    errors.append(
                        f"{file_path}:{line}: TEST_P '{suite}.{test_name}' does not have a matching INSTANTIATE_TEST_SUITE_P or GTEST_ALLOW_UNINSTANTIATED_PARAMETERIZED_TEST."
                    )

            # === Validate INSTANTIATE_TEST_SUITE_P references an existing TEST_P suite ===
            for suite, inst_map in instantiations.items():
                # If there is no TEST_P for this suite, then flag an error for each instantiation occurrence.
                if not any(suite == tp_suite for (tp_suite, _) in test_p_definitions.keys()):
                    for instantiation_name, line in inst_map.items():
                        errors.append(
                            f"{file_path}:{line}: INSTANTIATE_TEST_SUITE_P references non-existent TEST_P suite '{suite}'."
                        )

    return errors


def main():
    errors = analyze_tests(FOLDER_PATH)

    if errors:
        logging.error("The following issues were found:")
        for error in errors:
            logging.error(f"  {error}")
        raise ValueError("Validation failed. See the errors above.")
    else:
        logging.info("All tests meet the criteria.")


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
###############################################################################
#
# MIT License
#
# Copyright (c) 2024 Advanced Micro Devices, Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#################################################################################

"""gtest name linter"""
import os
import re
import sys
import argparse
from collections import defaultdict
import logging

logging.basicConfig(level=logging.WARNING)
logger = logging.getLogger("GTest name checker")

"""regexp based on https://github.com/ROCm/MIOpen/wiki/GTest-development#naming"""
re_prefix = re.compile(r"^((Smoke.*)|(Full.*)|(Perf.*)|(Unit.*))$")
re_hw = re.compile(r"^((CPU)|(GPU))$")
re_datatype = re.compile(
    r"^((FP((8)|(16)|(32)|(64)))|(BFP((8)|(16)))|(I((8)|(16)|(32)|(64)))|(NONE))\.?$"
)


def parse_args():
    """Function to parse cmd line arguments"""
    parser = argparse.ArgumentParser()

    parser.add_argument(
        "--list",
        dest="list",
        type=str,
        required=True,
        help="Specify gtest test list file",
    )
    args = parser.parse_args()

    return args


def check_naming_schema(args):

    mismatches = defaultdict(str)

    with open(args.list) as fp:
        for line in fp.readlines()[2:]:
            if not line.strip():
                continue
            if line[0] == " ":
                continue
            line = line.split("#")[0].strip()

            full_name = line.split("/")

            if len(full_name) == 2:
                prefix = re.search(re_prefix, full_name[0])
                name = full_name[1].split("_")

                # Try to recognize pattern like HW_NAME_TYPE/<number>. This pattern is used in case of TYPED_TEST_SUITE
                if not prefix and full_name[1].replace(".", "").isnumeric():
                    name = full_name[0].split("_")
                    prefix = ["empty"]

            else:
                prefix = ["empty"]
                name = full_name[0].split("_")

            hw = re.search(re_hw, name[0])
            datatype = re.search(re_datatype, name[-1])
            if not prefix:
                mismatches[line] += " Prefix"
            if not hw:
                mismatches[line] += " Hw"
            if not datatype:
                mismatches[line] += " Datatype"
            if hw and hw.group() == "GPU" and datatype and ("NONE" in datatype.group()):
                mismatches[line] += " Hw and Datatype combination (GPU+NONE)"

        for l, k in mismatches.items():
            logger.warning("Name: " + l + " Mismatch types:" + k)

        if mismatches:
            logger.critical(
                "Tests do not match to the test naming scheme (see https://github.com/ROCm/MIOpen/wiki/GTest-development#naming )"
            )
            return -1  # uncomment when all the tests will be renamed
    return 0

# This function makes sure that we don't have explicit name conflicts in gtest folder
# For example if you have GPU_SomeTestName_FP32 in file1.cpp and same GPU_SomeTestName_FP32 in file2.cpp
# in gtest folder, there will be a naming conflict when both files are combined into one single test binary miopen_gtest
# If such a situation is detected we should force a developer to make proper unique naming for the tests in PR.

# This script should be located in gtest folder
def check_names_uniqueness() :
    dir_path = os.path.dirname(os.path.realpath(__file__))
    files = os.listdir(dir_path)

    test_regexp = re.compile(r"^\s*(TEST)|(TEST_P)|(TEST_F)|(TYPED_TEST)\(.*,.*")

    occurences = {}

    error_count = 0
    files_count = 0
    cases_count = 0

    for file_name_raw in files:
        file_name = os.path.join(dir_path, file_name_raw)
        if not os.path.isfile(file_name):
            continue

        if not file_name.endswith(".cpp"):
            continue

        files_count += 1
        with open(file_name) as f:
            for line in f:
                if re.match(test_regexp, line):
                    m = re.search('\((.*),', line)
                    if (m is None):
                        continue

                    test_class_name = m.group(1).strip(" \t")

                    if not test_class_name in occurences:
                        occurences[test_class_name] = set()
                    occurences[test_class_name].add(file_name_raw)

    for key in occurences.keys():
        if len(occurences[key]) > 1:
            print ("ERROR: test name " + key + " is used in multiple files: " + str(occurences[key]))
            error_count += 1
            cases_count += len(occurences[key])

    print ("Gtest folder test class names uniqueness check, total cpp-files checked: " + str(files_count) + ", total errors = " + str(error_count) + ". Total files with duplicates = " + str(cases_count))

    if error_count > 0:
        return -1

    return 0

def main():
    """Main function"""
    args = parse_args()
    naming_check_result = check_naming_schema(args)

    if naming_check_result < 0:
        return naming_check_result

    return check_names_uniqueness()

if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python3
###############################################################################
#
# MIT License
#
# Copyright (c) 2023 Advanced Micro Devices, Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#################################################################################
"""Performance tracking script"""
import os
import re
import subprocess
import argparse
import csv
import time
from decimal import Decimal
import multiprocessing as mp

curr_path = os.path.abspath(os.getcwd())
default_results_path = curr_path + "/perf_results"
TOLERANCE = -10  #tolerance 10%

re_Elapsed = re.compile(r"(\d*\.*\d+)")
re_Solver = re.compile(r"^MIOpen .* Algorithm: (\d+), Solution: (\d+)/(\w+)")
re_GPU = re.compile(r"^GPU Kernel Time .* Elapsed: (\d+\.\d+) ms")
re_Key = re.compile(r"^.*Key match: ([\w\-]*)")

queue = mp.Queue()  #gpu idx queue, used in HIP_VISIBLE_DEVICES
results_queue = mp.Queue()


class Manager(mp.Process):
  """Queue manager"""

  def __init__(self, **kwargs):
    allowed_keys = set(['filename', 'install_path', 'override', 'results_path'])
    self.filename = None
    self.install_path = None
    self.override = False
    self.results_path = f"{default_results_path}"
    self.__dict__.update(
        (key, value) for key, value in kwargs.items() if key in allowed_keys)

    self.in_qs = {}
    self.num_gpus = int(self.get_num_gpus())
    self.resfile = f"{self.results_path}/{self.filename}"
    print(self.resfile)
    print('install_path: ', self.install_path)
    self.model_path = f"{self.install_path}/share/miopen/perf_models/{self.filename}"
    self.driver_cmds = []
    self.set_driver_cmds()
    self.writer = None

  def get_num_gpus(self):
    """Get num_gpus"""
    cmd = "/opt/rocm/bin/rocminfo | grep 'Device Type: *GPU' | wc -l"
    proc = subprocess.Popen(cmd,
                          shell=True,
                          stdout=subprocess.PIPE,
                          stderr=subprocess.STDOUT)
    output = proc.communicate()[0]
    return int(output.decode('utf-8').strip())

  def set_driver_cmds(self):
    if self.override:
      var_list = self.override.split(',')
      var_str = ""
      for var in var_list:
        var_str += var + " &&"

    with open(os.path.expanduser(self.model_path), "r",
              encoding='utf-8') as infile:
      for line in infile:
        try:
          if line.find('MIOpenDriver') == -1:
            print(f"Skipping line '{line}'")
            continue
          idx = line.index('MIOpenDriver')
          driver_cmd = line[idx:-1]
          if self.override:
            cmd = f"export LD_LIBRARY_PATH={self.install_path}/lib && export MIOPEN_LOG_LEVEL=6 && "\
                  f"export MIOPEN_SYSTEM_DB_PATH={self.install_path}/share/miopen/db && "\
                  f"export HIP_VISIBLE_DEVICES=GPU_ID && "\
                  f"{var_str} "\
                  f"{self.install_path}/bin/{driver_cmd} -V 0 -i 10 -w 1 -t 1 -G 1"
          else:
            cmd = f"export LD_LIBRARY_PATH={self.install_path}/lib && export MIOPEN_LOG_LEVEL=6 && "\
                  f"export MIOPEN_SYSTEM_DB_PATH={self.install_path}/share/miopen/db && "\
                  f"export HIP_VISIBLE_DEVICES=GPU_ID && "\
                  f"{self.install_path}/bin/{driver_cmd} -V 0 -i 10 -w 1 -t 1 -G 1"
          print(f'Appending cm: {cmd}')
          self.driver_cmds.append(cmd)

        except Exception as err:
          print(f"Could not get driver commands: {err}")

    print('#driver commands: ', len(self.driver_cmds))

  def run(self):
    """Main function to launch worker pool"""
    for gpu_id in range(self.num_gpus):
      queue.put(gpu_id)
    self.launch_pool()

  def write_to_file(self, results):
    """Write results to csv file"""
    field_names = [
        'Driver', 'k_time', 'wall_time', 'solver_id', 'solver_name', 'fdb_key'
    ]
    with open(os.path.expanduser(self.resfile), 'w+', encoding='utf-8') as outfile:
      self.writer = csv.DictWriter(outfile, fieldnames=field_names)
      self.writer.writeheader()
      try:
        self.writer.writerows(results)
        print(f"Perf results written to: {self.resfile}")
      except Exception as exp:
        print(exp)

  def launch_pool(self):
    """Launch pool of driver cmds"""
    pool = mp.Pool(processes=self.num_gpus)
    for result in pool.imap_unordered(self.run_driver_cmd, self.driver_cmds):
      self.parse_result(result)
    pool.close()
    print(f"Size of results Q: {results_queue.qsize()}")
    results = []
    while not results_queue.empty():
      results.append(results_queue.get())
    results.sort(key=lambda x: x['Driver'])
    self.write_to_file(results)

  def parse_result(self, result):
    """Potential to use result as it becomes available"""
    #print(result)

  def run_driver_cmd(self, driver_cmd):
    """Launch each driver cmd in subproc"""
    while queue.empty():
      time.sleep(2)
      print('GPUs busy, sleeping')
    gpu_id = queue.get()
    cmd = driver_cmd.replace('GPU_ID', str(gpu_id))
    try:
      print("")
      print(f"Starting process on GPU {gpu_id}")
      print(cmd)

      res_dict = {}
      proc = subprocess.Popen(cmd,
                              shell=True,
                              stdout=subprocess.PIPE,
                              stderr=subprocess.STDOUT)
      p_out = proc.stdout.readlines()
      res = None
      e = Entry()
      for line in p_out:
        line = line.decode("utf-8")
        line = line.strip()
        if (line.find('MIOpenDriver') != -1) and (line.find('MIOpen(HIP)') == -1):  #fragile solution
          e.cmd = line
          print(e.cmd)
          continue
        if line.find('Wall-clock Time') != -1:
          res = re_Elapsed.findall(line)
          print(res)
          e.wall_elapsed = res[0]
          e.wall_aux = res[1]
          e.wall_gwss = res[2]
          continue
        if re_Solver.match(line):
          res = re_Solver.findall(line)[0]
          print(res)
          e.algo = res[0]
          e.sol_id = res[1]
          e.sol_name = res[2]
          continue
        if re_Key.match(line):
          e.fdb_key = re_Key.findall(line)[0]
        if re_GPU.match(line):
          res = re_GPU.findall(line)
          e.sol_time = res[0]
          print('k_time: ', e.sol_time)
        if line.find('error') != -1:
          raise ValueError(p_out)

      res_dict = {
        'Driver': e.cmd,
        'k_time': e.sol_time,
        'wall_time': e.wall_elapsed,
        'solver_id': e.sol_id,
        'solver_name': e.sol_name,
        'fdb_key': e.fdb_key
      }

      results_queue.put(res_dict)
      ret = res_dict
    finally:
      queue.put(gpu_id)
    return ret


class Entry:
  """Module to hold runtime values"""

  def __init__(self):
    self.cmd = ''
    self.wall_elapsed = ''
    self.wall_aux = ''
    self.wall_gwss = ''
    self.algo = ''
    self.sol_id = ''
    self.sol_name = ''
    self.sol_time = ''
    self.fdb_key = ''

  def __str__(self):
    atrs = [
        self.cmd, self.wall_elapsed, self.wall_aux, self.wall_gwss, self.algo,
        self.sol_id, self.sol_name, self.sol_time, self.fdb_key
    ]
    return ",".join(atrs)


def parse_args():
  """Function to parse cmd line arguments"""
  parser = argparse.ArgumentParser()
  group1 = parser.add_mutually_exclusive_group()

  parser.add_argument('--filename',
                      dest='filename',
                      type=str,
                      required=True,
                      help='Specify filename for model')
  group1.add_argument('--install_path',
                      dest='install_path',
                      help='Specify MIOpenDriver install path')
  group1.add_argument('--compare_results',
                      dest='compare_results',
                      action='store_true',
                      help='Compare current perf results to old perf results')
  parser.add_argument('--old_results_path',
                      dest='old_results_path',
                      type=str,
                      help='Specify full path to old results directory')
  parser.add_argument('--results_path',
                      dest='results_path',
                      default=default_results_path,
                      type=str,
                      help='Specify full path to output results directory')
  parser.add_argument('--override',
                      dest='override',
                      type=str,
                      help='Specify driver cmd env vars')
  args = parser.parse_args()

  if args.compare_results and not args.old_results_path:
    parser.error(
        '--old_results_path and --compare_results must both be specified')

  return args


def compare_results(args):
  """Compare current results with previous results"""
  if not os.path.exists(args.results_path):
    raise ValueError(f"Results path does not exist {args.results_path}")
  if not os.path.exists(args.old_results_path):
    raise ValueError(f"Old results path does not exist {args.old_results_path}")

  if not compare_file(f"{args.results_path}/{args.filename}", \
    f"{args.old_results_path}/{args.filename}"):
    raise ValueError(f"FAILED: {args.filename}")
  print(f"PASSED: {args.filename}")


def compare_file(new_results, old_results):
  """Compare kernel_time in new vs old results file"""
  ret = True
  print(f"Comparing results from: {new_results} to {old_results}")
  with open(new_results,
            'r', encoding='utf-8') as new, open(old_results,
                                                'r',
                                                encoding='utf-8') as old:
    csv_new = list(csv.DictReader(new))
    csv_new.sort(key=lambda x: x['Driver'])
    csv_old = list(csv.DictReader(old))
    csv_old.sort(key=lambda x: x['Driver'])
    for line_new, line_old in zip(csv_new, csv_old):
      if line_new['Driver'] != line_old['Driver']:
        print(f"New driver: {line_new['Driver']}")
        print(f"Old driver: {line_old['Driver']}")
        raise ValueError('Result files are out of sync')
      if line_new['solver_name'] != line_old['solver_name']:
        print(
            f"Winning solver changed from {line_new['solver_name']} to {line_old['solver_name']}"
        )
      speedup = (Decimal(line_old['k_time']) - Decimal(
          line_new['k_time'])) / Decimal(line_old['k_time']) * 100
      if int(speedup) < TOLERANCE:
        print(f"{line_new['Driver']}")
        print(f"Speedup: {speedup}%")
        print(
            f"FAILED for new k_time: {line_new['k_time']} - old k_time: {line_old['k_time']}\n"
        )
        ret = False

  return ret


def main():
  """Main function"""
  args = parse_args()

  if args.compare_results:
    try:
      compare_results(args)
    except Exception as ex:
      print(f'ERR: {ex}')
      return False
  else:
    if not os.path.exists(args.results_path):
      os.makedirs(args.results_path)

    try:
      manager = Manager(filename=args.filename,
                        install_path=args.install_path,
                        override=args.override,
                        results_path=args.results_path)
      manager.run()
    except Exception as ex:
      print(f'ERR: {ex}')
      return False

  return True


if __name__ == '__main__':
  main()
